<?php

/**
 * Geral\Repositories
 *
 * @category Php
 * @package  Geral\Repositories
 * @author   setforms <setforms@setforms.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
namespace Geral\Repositories;

use \Geral\Util\DateUtil;

/**
 * Class Itemsoli
 *
 * @category Php
 * @package  Geral\Repositories
 * @author   setforms <setforms@setforms.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
class Itemsoli extends \Doctrine\ORM\EntityRepository
{
    /**
     * Insere todos os produtos dentro do intervalo de PRODINI e PRODFIN na tabela de ITEMSOLI, desde que estes ainda
     * nao existam na tabela.
     *
     * @param array $params Params
     *
     * @return \Doctrine\DBAL\Driver\Statement
     * @throws \Exception
     */
    public function insereListaDeItens(array $params)
    {
        $where = " ";

        $sql = "
          INSERT INTO ITEMSOLI (
            CDFILSOLI,
            CDFILLAN,
            NRSOLICMP,
            CDPRODUTO,
            DTPROGENTR,
            QTSOLIPROD,
            IDITMSOLEXTRA,
            NRORG,
            DTINCLUSAO,
            NRORGINCLUSAO,
            CDOPERINCLUSAO
          )
          (
            SELECT :CDFILSOLI, :CDFILLAN, :NRSOLICMP, CDPRODUTO, to_date(:DTPROGENTR, 'DD/MM/YYYY'), :QTSOLIPROD,
                :IDITMSOLEXTRA, :NRORG, SYSDATE, :NRORG, :CDOPERINCLUSAO
              FROM PRODUTO
              WHERE (CDARVPROD BETWEEN :PRODINI AND :PRODFIN)
                AND (CDNVPRODUTO = :NVPROD)
                AND CDPRODUTO NOT IN (
                  SELECT CDPRODUTO
                    FROM ITEMSOLI
                    WHERE NRORG       = :NRORG
                      AND (NRSOLICMP  = :NRSOLICMP)
                      AND (CDFILSOLI  = :CDFILSOLI)
                      AND (CDFILLAN   = :CDFILLAN)
                      AND (DTPROGENTR = :DTPROGENTR)
                )
              AND (CDARVPROD BETWEEN :PRODINI AND :PRODFIN)
              AND (CDPRODUTO != CDPRODESTO OR :PRODUTOCOMPRA = 'N')

              $where
          )
        ";

        $parametros = array(
            'CDFILSOLI'      => $params['CDFILSOLI'],
            'CDFILLAN'       => $params['CDFILLANC'],
            'NRSOLICMP'      => $params['NRSOLICMP'],
            'DTPROGENTR'     => DateUtil::getDataDeString($params['DTPROGENTR'])->format(DateUtil::FORMATO_BRASILEIRO),
            'QTSOLIPROD'     => $params['QTSOLIPROD'],
            'IDITMSOLEXTRA'  => $params['IDITMSOLEXTRA'],
            'NRORG'          => $params['NRORG'],
            'CDOPERINCLUSAO' => $params['CDOPERINCLUSAO'],
            'PRODINI'        => $params['CDPRODINI'],
            'PRODFIN'        => $params['CDPRODFIN'],
            'NVPROD'         => $params['NVPROD'],
            'PRODUTOCOMPRA'  => $params['PRODUTOCOMPRA'],
        );

        try {
            return $this->getEntityManager()->getConnection()->executeQuery($sql, $parametros);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Executa a importacao de uma solicitacao: Copia todos os itens de determinada solicitacao para uma nova.
     *
     * @param array $params Params
     *
     * @return \Doctrine\DBAL\Driver\Statement
     * @throws \Exception
     */
    public function importaSolicitacao(array $params)
    {
        $sql = "
        INSERT INTO ITEMSOLI (
          CDFILSOLI,
          CDFILLAN,
          NRSOLICMP,
          CDPRODUTO,
          DTPROGENTR,
          QTSOLIPROD,
          IDITMSOLEXTRA,
          NRORG,
          DTINCLUSAO,
          NRORGINCLUSAO,
          CDOPERINCLUSAO
        )
        (
          SELECT DISTINCT
            :CDFILSOLI,
            :CDFILLAN,
            :NRSOLICMP,
            CDPRODUTO,
            :DTPROGENTR,
            :QTSOLIPROD,
            :IDITMSOLEXTRA,
            :NRORG,
            :DTINCLUSAO,
            :NRORG,
            :CDOPERINCLUSAO
          FROM ITEMSOLI
          WHERE NRORG       = :NRORG
          AND NRSOLICMP     = :NRSOLICMPIMP
          AND CDFILSOLI     = :CDFILSOLIIMP
          AND CDFILLAN      = :CDFILLANIMP
          AND CDPRODUTO NOT IN (
            SELECT CDPRODUTO
              FROM ITEMSOLI
              WHERE NRORG      = :NRORG
                AND NRSOLICMP  = :NRSOLICMP
                AND CDFILSOLI  = :CDFILSOLI
                AND CDFILLAN   = :CDFILLAN
                AND DTPROGENTR = :DTPROGENTR
          )
        )
        ";

        $parametros = array(
            "CDFILSOLI"      => $params['CDFILSOLIMP'],
            "CDFILLAN"       => $params['CDFILLANC'],
            "NRSOLICMP"      => $params['NRSOLICMP'],
            "DTPROGENTR"     => $params['DTPROGENTR'],
            "QTSOLIPROD"     => $params['QTSOLIPROD'],
            "IDITMSOLEXTRA"  => $params['IDITMSOLEXTRA'],
            "NRORG"          => $params["NRORG"],
            "DTINCLUSAO"     => $params['DTINCLUSAO'],
            "CDOPERINCLUSAO" => $params['CDOPERINCLUSAO'],
            "NRSOLICMPIMP"   => $params['NRSOLICMPIMP'],
            "CDFILSOLIIMP"   => $params['CDFILSOLIMP'],
            "CDFILLANIMP"    => $params['CDFILLANCIMP']
        );

        try {
            return $this->getEntityManager()->getConnection()->executeQuery($sql, $parametros);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Remove itens de uma solicitacao dentro de uma faixa de produtos.
     *
     * @param String $nrorg      Nrorg
     * @param String $cdfillanca Cdfillanca
     * @param String $cdfilsoli  Cdfilsoli
     * @param String $nrsolicmp  Nrsolicmp
     * @param String $cdprodini  Cdprodini
     * @param String $cdprodfin  Cdprodfin
     *
     * @return \Doctrine\DBAL\Driver\Statement
     * @throws \Exception
     */
    public function deletaListaProdutos($nrorg, $cdfillanca, $cdfilsoli, $nrsolicmp, $cdprodini, $cdprodfin)
    {
        $sql = "
            DELETE
            FROM ITEMSOLI
            WHERE NRORG    = :NRORG
            AND CDFILSOLI  = :CDFILSOLI
            AND (CDFILLAN  = :CDFILLAN)
            AND (NRSOLICMP = :NRSOLICMP)
            AND CDPRODUTO IN
              (SELECT CDPRODUTO
              FROM PRODUTO
              WHERE NRORG = :NRORG
              AND (CDARVPROD BETWEEN :CDPRODINI AND :CDPRODFIN))
        ";

        $params = array(
            "NRORG"     => $nrorg,
            "CDFILLAN"  => $cdfillanca,
            "CDFILSOLI" => $cdfilsoli,
            "NRSOLICMP" => $nrsolicmp,
            "CDPRODINI" => $cdprodini,
            "CDPRODFIN" => $cdprodfin,
        );

        try {
            return $this->getEntityManager()->getConnection()->executeQuery($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Remove todos os produtos de uma solicitacao que estejam dentro de um intervalo de datas.
     *
     * @param String $cdfilsoli  Cdfilsoli
     * @param String $cdfillanca Cdfillanca
     * @param String $nrsolicmp  Nrsolicmp
     * @param String $nrorg      Nrorg
     * @param String $dtinicial  Dtinicial
     * @param String $dtfinal    Dtfinal
     *
     * @return \Doctrine\DBAL\Driver\Statement
     * @throws \Exception
     */
    public function deletarProdutosPorIntervaloDeDatas($cdfilsoli, $cdfillanca, $nrsolicmp, $nrorg, $dtinicial, $dtfinal)
    {
        $sql = "
            DELETE FROM ITEMSOLI
              WHERE CDFILLAN  = :CDFILLAN
                AND CDFILSOLI = :CDFILSOLI
                AND NRSOLICMP = :NRSOLICMP
                AND NRORG     = :NRORG
                AND DTPROGENTR BETWEEN TO_DATE(:DTINICIAL, 'DD/MM/YYYY') AND TO_DATE(:DTFINAL, 'DD/MM/YYYY')
        ";

        $params = array(
            "CDFILLAN"  => $cdfillanca,
            "CDFILSOLI" => $cdfilsoli,
            "NRSOLICMP" => $nrsolicmp,
            "NRORG"     => $nrorg,
            "DTINICIAL" => DateUtil::getDataDeString($dtinicial)->format(DateUtil::FORMATO_BRASILEIRO),
            "DTFINAL"   => DateUtil::getDataDeString($dtfinal)->format(DateUtil::FORMATO_BRASILEIRO),
        );

        try {
            return $this->getEntityManager()->getConnection()->executeQuery($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Altera o campo de IDSOLIAPROV da tabela de ITEMSOLI, segundo os parametros passados. Este metodo permite que a
     * alteracao de status seja feita a nivel de solicitacao(todos os itens de uma determinada solicitacao terao seu
     * status alterado para o mesmo valor), caso os parametros de cdproduto e dtprogentr nao sejam informados, ou a
     * nivel de item, passando todos os parametros.
     *
     * @param String $idsoliaprov Idsoliaprov
     * @param String $cdfilsoli   Cdfilsoli
     * @param String $cdfillan    Cdfillan
     * @param String $nrsolicmp   Nrsolicmp
     * @param String $nrorg       Nrorg
     *
     * @return \Doctrine\DBAL\Driver\Statement
     * @throws \Exception
     */
    public function alteraStatusDeAprovacao($idsoliaprov, $cdfilsoli, $cdfillan, $nrsolicmp, $nrorg)
    {
        $params = array(
            "IDSOLIAPROV" => $idsoliaprov,
            "CDFILSOLI"   => $cdfilsoli,
            "CDFILLAN"    => $cdfillan,
            "NRSOLICMP"   => $nrsolicmp,
            "NRORG"       => $nrorg,
        );

        $sql = "
            UPDATE ITEMSOLI
              SET IDSOLIAPROV = :IDSOLIAPROV
              WHERE CDFILSOLI = :CDFILSOLI
                AND CDFILLAN  = :CDFILLAN
                AND NRSOLICMP = :NRSOLICMP
                AND NRORG     = :NRORG
        ";

        try {
            return $this->getEntityManager()->getConnection()->executeQuery($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Remove todos os produtos selecionados de uma certa data.
     *
     * @param String $cdfilsoli  Cdfilsoli
     * @param String $cdfillanca Cdfillanca
     * @param String $nrsolicmp  Nrsolicmp
     * @param String $nrorg      Nrorg
     * @param String $itens      Itens
     *
     * @return \Doctrine\DBAL\Driver\Statement
     * @throws \Exception
     */
    public function deletaItemsoli(
        $cdfilsoli, $cdfillanca, $nrsolicmp, $nrorg, $itens
    )
    {

        $sql = "
            DELETE FROM ITEMSOLI
              WHERE CDFILLAN   = :CDFILLAN
                AND CDFILSOLI  = :CDFILSOLI
                AND NRSOLICMP  = :NRSOLICMP
                AND NRORG      = :NRORG
                AND INSTR (:ITENS, CDPRODUTO) <> 0
        ";

        $params = array(
            "CDFILLAN"  => $cdfillanca,
            "CDFILSOLI" => $cdfilsoli,
            "NRSOLICMP" => $nrsolicmp,
            "NRORG"     => $nrorg,
            "ITENS"     => $itens,
        );

        try {
            return $this->getEntityManager()->getConnection()->executeQuery($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function deleteItemsoli($cdfilsoli,$cdfillan,$nrsoli,$dtprogen){
        try{
            $sql = "DELETE ITEMSOLI WHERE NRSOLICMP = '$nrsoli' AND CDFILSOLI = '$cdfilsoli' AND CDFILLAN = '$cdfillan' AND DTPROGENTR='$dtprogen'";
            return $this->getEntityManager()->getConnection()->executeQuery($sql);
        }catch(\Exception $e){
            throw $e;
        }
    }
}