<?php
/**
 * PercorreCalculo.
 *
 * @package Logic
 *
 * @author  alessandrobartels
 */
class PercorreCalculo{

    private $_parametros;
    private $_nrOrgPadrao;
    private $_nrOrgTrabalho;
    private $_nrEstruturaTrab;

    private $_idDebug;
    private $_ambienteSalarioOperador;
    private $_nrTipoModalidadeCalculo;
    private $_nrOcorrenciaCalculo;

    private $_mesCompetencia;
    private $_nrTipoCalculo;
    private $_simulacao;
    private $_retrocesso;
    private $_parcial;
    private $_dtPagamento;
    private $_ultPagamento;
    private $_dt_inicio;
    private $_dt_fim;
    private $_dsInicio;
    private $_dsFim;
    private $_id_retrocesso;
    private $_dsOrdem;
    private $_tipo_calculo;  /* nao usado no codigo */
    private $_calculo_folha;
    /** @var $_tp_calc_mov FpatpcalcmovEntity */
    private $_tp_calc_mov;
    /** @var $_vinculom GpevinculomEntity */
    private $_vinculom;
    private $_tpVinculoProcesso;
    private $_tpRescisaoDireito;
    private $_pessoa; /* nao usado no codigo */
    private $_dependencia;


//    private $_tipo_dependencia;
    private $_formulario;


    /**
     * Construtor.
     *
     * @param integer    $nr_tipo_calculo  Numero do Tipo de Calculo.
     * @param string     $tp_processamento Tipo de Processamento.
     * @param integer    $nrFormulario     Numero do Formulario.
     * @param Parametros $parametro        Instancia de Parametros.
     *
     * @throws Exception $e Propaga Excecao.
     *
     * @autor alessandrobartels
     *        alessandrobartels - 05/07/2011 - Retirado nrOrg e cdOperador do parametro. Setadas do Ambiente no Logic.
     *        alessandrobartels - 27/12/2011 - Modificada parametrizacao. Dados do formulario provem agora da instancia.
     */
    public function __construct($nr_tipo_calculo, $tp_processamento, $nrFormulario, Parametros $parametro){

        $this->_nrTipoCalculo = $nr_tipo_calculo;

        $this->_parametros = $parametro;
        $this->_nrOrgTrabalho = $this->_parametros->getNrOrgTrabalho();
        $this->_nrOrgPadrao = $this->_parametros->getNrOrgPadrao();
        $this->_nrEstruturaTrab = \FPA::getEstruturaTrabalho();
//        $this->_mesCompetencia = $this->_parametros->getMesCompetencia();
        $this->_mesCompetencia = $this->_parametros->getMesCompetencia();

        $this->_nrTipoModalidadeCalculo = $this->_parametros->getTipoModalidadeCalculo();
        $this->_nrOcorrenciaCalculo = $this->_parametros->getNrOcorrenciaCalculo();


        $this->_buscaDadosFormulario($nrFormulario);

        /* $this->_ambienteSalarioOperador = $parametro->ambienteSalarioOperador();
        $this->_idDebug = $parametro->getIdDebug();
        */

        /* Retorna a data para o primeiro dia do mes
        $this->_mesCompetencia =  new TkDateTime($mes_competencia, TkDateTime::FORMAT_DATE);
        $this->_mesCompetencia->setDay(1);
        */

        /* $ this->_dt_pagamento = $dt_pagamento; OK*/
        // #####$this->_ultPagamento = $ultPagamento;
        // #####$this->_id_retrocesso = $id_retrocesso;
        /*$this->_dt_inicio = $dt_inicio;
        $this->_dt_fim = $dt_fim;*/
        // $this->_dsOrdem = $dsOrdem; OK.
        $this->_retrocesso = false;
        $this->_parcial = false;

        /* Tipo processamento */
        $tp_processamento = strtoupper($tp_processamento);
        switch ($tp_processamento){
            case Parametros::PROC_SIMULACAO :
                $this->_simulacao = true; break;
            case Parametros::PROC_CALCULO :
                $this->_simulacao = false; break;
            case Parametros::PROC_RETROCESSO :
                $this->_retrocesso = true; break;
            case Parametros::PROC_CALCULO_PARCIAL :
                $this->_simulacao = false;
                $this->_parcial = true; break;
            default:
                tk_message('Tipo de processo '.$tp_processamento.' invalido', 'warning');
        }

        /* Parametrizadas as datas no Container para serem utilizadas no FpaeventomEntity */
        Inicializa::Container($this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia, $this->_dtPagamento);
        Inicializa::carregaVariaveis();

        $this->_vinculom = TkDI::getContainer()->vinculom;
        $this->_tipo_calculo = TkDI::getContainer()->tipo_calculo; // Nao utilizado.
        $this->_calculo_folha = TkDI::getContainer()->calculo_folha;
        $this->_pessoa = TkDI::getContainer()->pessoa;  // Nao utilizado.
        $this->_tp_calc_mov = TkDI::getContainer()->tp_calc_mov;

        $this->_dependencia = new Dependencia($this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia);
        $this->_tpVinculoProcesso = new FpaTpvincprocessoEntity();
        $this->_tpRescisaoDireito = new FpaDireitotpdemisEntity();
//        $this->_dependencia = TkDI::getContainer()->dependencia;
//        $this->_tipo_dependencia = TkDI::getContainer()->tipo_dependencia;

    }


    /**
     * Enter description here ...
     *
     * @throws Exception Tipo Calculo Movimento retorna null.
     *
     * @return multitype:
     *
     * @author alessandrobartels -
     *         alessandrobartels - 04/07/2011 - Retirado $demonstrativo do Facade. Instanciado no percorre().
     *         alessandrobartels - 23/08/2011 - $mesCompetencia parametrizado passa a ser feito pelo construtor.
     *         alessandrobartels - 27/12/2011 - Modificada parametrizacao.Dados do formulario provem agora da instancia.
     */
    public function percorre(){

        $objTipoCalculo = new FpatipocalculoEntity();
        $objTpCalc = $objTipoCalculo->retornaTipoCalculo($this->_nrTipoCalculo);
        //tk_warning($this->_nrTipoCalculo.' | '.$objTpCalc->getNmtipocalculo());
        $demonstrativo = new Demonstrativo(
            $objTpCalc->getNmtipocalculo(), $this->_mesCompetencia->toString(TkDateTime::FORMAT_DATE)
        );

        try {
            $this->_tp_calc_mov->retornaTipoCalculoMovimento($this->_nrTipoCalculo);  ///
        } catch (Exception $e){
            throw $e;
        }


        $this->_tp_calc_mov->atribuiNrCalculoFolha(
            $this->_nrOrgTrabalho, $this->_calculo_folha, $this->_mesCompetencia, $this->_nrTipoModalidadeCalculo,
            $this->_nrOcorrenciaCalculo
        );
        //  BORBA 26/12/2014 testa também se é rescisao complementar
        if ($this->_nrTipoCalculo === FpatipocalculoEntity::CALC_RESCISAO ||
            $this->_nrTipoCalculo === FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR)  {
			
            if ($this->_nrTipoCalculo === FpatipocalculoEntity::CALC_RESCISAO) {
                $vinculos = $this->_vinculom->retornaVinculosDemitidosMesCompetencia(
                    $this->_nrOrgTrabalho, $this->_mesCompetencia, $this->_dsInicio, $this->_dsFim,
                    $this->_dt_inicio, $this->_dt_fim, $this->_dsOrdem, $this->_nrEstruturaTrab
                );
            };

			
			//Marcelo 21-11-2014 Alteração da função destinada a buscar o vínculo para trazer todos os vinculos rescindidos
			//independente do mes de competencia
            if ($this->_nrTipoCalculo === FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR) {
                $vinculos = $this->_vinculom->retornaVinculosDemitidosGeral(
                    $this->_nrOrgTrabalho, $this->_mesCompetencia, $this->_dsInicio, $this->_dsFim,
                    $this->_dt_inicio, $this->_dt_fim, $this->_dsOrdem, $this->_nrEstruturaTrab
                );
            };

            $this->_tpRescisaoDireito->carregaTipoDireitoPorTipoDemissao(
                $this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia
            );

        } else {
            $vinculos = $this->_vinculom->retornaVinculosAtivosMesCompetencia(
                $this->_nrOrgTrabalho, $this->_mesCompetencia, $this->_dsInicio, $this->_dsFim, $this->_dsOrdem,
                false, $this->_nrEstruturaTrab
            );
        }

        // Carrega os tipos de vinculos por processo
        $this->_tpVinculoProcesso->carregaTipoVinculoPorProcesso(
            $this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia
        );

        /*$calculo = new Calculo(
            $this->_parametros, $this->_nrTipoCalculo, $this->_simulacao, $this->_parcial, $this->_dt_pagamento,
            $this->_id_retrocesso, $this->_mesCompetencia, $this->_ultPagamento
        );*/

        /*$preparacao = new Preparacao(
            $this->_parametros, $this->_nrOrgPadrao, $this->_nrOrgTrabalho, $this->_nrTipoCalculo,
            $this->_mesCompetencia, false, $this->_dt_inicio, $this->_dt_fim, false, $this->_dt_pagamento
        );*/

        $qtd_vinculo = count($vinculos);
        if ($qtd_vinculo > 0) {
            $resultado = array();
            $contador = 0;
            $idstatus = MsgCalculo::ID_PENDENTE;
            $idabort = false;

            $contMax = ($this->_simulacao==true) && ($qtd_vinculo > 10) ? 10 : $qtd_vinculo;

			$calculo = new Calculo($this->_parametros, $this->_formulario, $this->_nrTipoCalculo, $this->_simulacao, $this->_parcial);

            /* A preparacao chamada dentro do calculo nunca apaga os itens (retrocede), por isso id_retrocesso eh false
             * A preparacao chamada dentro do calculo sempre persiste item caluclo por isso simulacao eh false
             */
			$preparacao = new Preparacao($this->_parametros, $this->_formulario, $this->_nrTipoCalculo, false, false);

            $process = new \Setforms2\Shared\Process\ProcessTracking();
            $process->startProcess('Calculo', $contMax, 'Vinculos');

            foreach ($vinculos as $vinculo) {
                $process->notifyRead();

                //@todo MsgCalculo Inicio rotina
                /*FelipeSantos 16/04/2015 - Rotina de processamento de erros/alertas do cálculo*/
                $processamento = MsgCalculo::getInstance();

                //se estiver simulando limita a quantidade de vinculos a 10 registros
                //leandroafonso 06/08
                //BORBA 28/07/2014 @todo definir quantidade de vinculos a serem simulados por parametro definido pelo operador.
                if($contador>10 && $this->_simulacao==true){
                    tk_warning("Simulado e demonstrado apenas os 10 primeiros vínculos da lista. Para calcular mais vínculos será necessário gravar.");
                    break;
                }
                if ($this->_retrocesso === false) {
//                    $result_parcial = array();

                    // Modificado para novo metodo para diminuir a complexidade
                    try {
                        $this->_executaPreparacao($vinculo, $preparacao, $demonstrativo, $contador);
                    } catch (\Folha\Exceptions\FeriasCalculo $e) {

                        tk_warning($e->getMessage());

                        continue;
                    }

                    // Analisar dependentes para IR e Salário Família.
                    $this->_dependencia->analisaDependencia($vinculo);
                    // @todo verificar a possibilidade de agregar novo historico ao vinculo

                        // Determina a lista de processos de acordo com o tipo de vinculo
                        $listaProcessos = $this->_tpVinculoProcesso->getItem($vinculo->getNrtipovinculom());


                        // Determina a lista de direitos de acordo com o tipo de rescisao
                         /*if ($this->_nrTipoCalculo === FpatipocalculoEntity::CALC_RESCISAO) {
                             $listaDireitos = $this->_tpRescisaoDireito->getItem($vinculo->getNrtpdemissao());
                             $calculo->setListaDireitos($listaDireitos);
                         }*/

                        // Testa os processos para o tipo de calculo
                        switch ($this->_nrTipoCalculo){
	                        case FpatipocalculoEntity::CALC_FERIAS:
	                            if (in_array(FpaProcessoEntity::FERIAS, $listaProcessos) === false){
	                                continue;
	                            }
	                            break;
	                        case FpatipocalculoEntity::CALC_13SAL:
	                        case FpatipocalculoEntity::CALC_ADIANT_13SAL:
	                            if (in_array(FpaProcessoEntity::DECIMO_TERCEIRO_SALARIO, $listaProcessos) === false){
	                                continue;
	                            }
	                            break;
	                        case FpatipocalculoEntity::CALC_ADIANTAMENTO:
	                            if (in_array(FpaProcessoEntity::ADIANTAMENTO_SALARIO, $listaProcessos) === false){
	                                continue;
	                            }
	                            break;
	                        case FpatipocalculoEntity::CALC_RESCISAO:
	                        case FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR:
	                            if (in_array(FpaProcessoEntity::RESCISAO, $listaProcessos) === false){
	                                continue;
	                            } else {
	                                $listaDireitos = $this->_tpRescisaoDireito->getItem($vinculo->getNrtpdemissao());
	                                $calculo->setListaDireitos($listaDireitos);
	                                if ($vinculo->getNrtpdemissao() == null){
	                                    //@todo MsgCalculo
	                                    MsgCalculo::getInstance()->adicionaMensagem('Vínculo sem tipo de demissão.');
	                                    //tk_warning('Vinculo sem tipo de demissão '.$vinculo->getNrvinculom());
	                                }
	                            }
	                        break;
	                        case FpatipocalculoEntity::CALC_PROVISAO_CONTABIL:
	                            if (in_array(FpaProcessoEntity::PROVISAO, $listaProcessos) === false){
	                                continue;
	                            }
	                            break;
                            case FpatipocalculoEntity::CALC_ADIANT_PLR:
                            case FpatipocalculoEntity::CALC_PLR:
                                if (in_array(FpaProcessoEntity::PLR, $listaProcessos) === false){
                                    continue;
                                }
                                break;
                        }

                        $calculo->setListaProcessos($listaProcessos);
                       // $result_parcial = $calculo->calcular($vinculo, $this->_tp_calc_mov, $demonstrativo, $contador);
                        $calculo->calcular($vinculo, $this->_tp_calc_mov, $demonstrativo, $contador);

                } else {
                    try {
                         $calculo->retroceder($vinculo, $this->_tp_calc_mov);
                    } catch (Exception $e){
                        tk_warning($e->getMessage());
                    }
                }

                //@todo MsgCalculo Fim rotina
                $msgcalculo = $processamento->getDados();

                if ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ERRO) {
                    $idstatus = MsgCalculo::ID_COM_ERRO;
                } elseif ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ALERTA and $idstatus != MsgCalculo::ID_COM_ERRO) {
                    $idstatus = MsgCalculo::ID_COM_ALERTA;
                }

                if ($msgcalculo['IDABORT'] === true) {
                    $idabort = true;
                    break;
                } else {
                    $processamento->salvaMensagens();
                }

                $processamento->clear();

                $process->notifyProcessed();
                $contador++;
                // jorgefonseca 30/05/2013
                \Setforms2\Runtime\Connection::getInstance()->clear();
            }//fim foreach

            //felipesantos 23/04/2015
            if ($idabort === false) {
                try {
                    $fpacalculofolha = \Folha\Entities\FpaCalculofolha::findOneByNrcalculofolhaAndNrorg($msgcalculo['NRCALCULOFOLHA'], $this->_nrOrgTrabalho);
                    if ($idstatus === MsgCalculo::ID_COM_ERRO) {
                        $fpacalculofolha->setIdstatusproc('E');
                    } else {
                        $fpacalculofolha->setIdstatusproc('C');
                    }
                    $fpacalculofolha->persist();
                    \Setforms2\Shared\Registry::getEntityManager()->flush();
                } catch (Exception $e) {
                    \Setforms2\Shared\Registry::getEntityManager()->rollback();
                    throw $e;
                }
            }

            $process->endProcess();
            // felipesantos 23/04/2015
            /* return $resultado; */
            return array(
                'DEMONSTRATIVO' => $demonstrativo,
                'CALCULO' => $msgcalculo['NRCALCULOFOLHA'],
                'MENSAGENS' => ($idabort === true ? $msgcalculo['MENSAGENS'] : null),
                'ERRO' => ($idstatus === MsgCalculo::ID_COM_ERRO),
                'ABORT' => $idabort
            );
        } else {
            return null;
        }
    }


    /**
     * Enter description here ...
     *
     * @param integer $nrFormulario Numero do formulario.
     *
     * @throws Exception Propaga excecao.
     *
     * @return void
     */
    private function _buscaDadosFormulario($nrFormulario){

        switch ($this->_nrTipoCalculo) {
        case FpatipocalculoEntity::CALC_FERIAS :
            $this->_buscaDadosFormularioFerias($nrFormulario);
            break;

        case FpatipocalculoEntity::CALC_RESCISAO :
            $this->_buscaDadosFormularioRescisao($nrFormulario);
            break;

        //BORBA 25/02/2014
        case FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR :
            $this->_buscaDadosFormularioRescocomplementar($nrFormulario);
            break;

        default:
            $this->_buscaDadosFormularioCalculo($nrFormulario);
            break;
        }

    //    $this->_dsInicio = FPARegistro::_trataDado($this->_dsInicio);
    //    $this->_dsFim = FPARegistro::_trataDado($this->_dsFim);

    }


    /**
     * Enter description here ...
     *
     * @param integer $nrFormulario Numero do formulario.
     *
     * @throws Exception Propaga excecao.
     *
     * @return void
     */
    private function _buscaDadosFormularioCalculo($nrFormulario){
        $formulario = new FpaformcalculoEntity();
        $this->_formulario = $formulario->retornaDadosFormularioCalculo($this->_nrOrgTrabalho, $nrFormulario);
        if (is_object($this->_formulario) === true) {
            $this->_dsOrdem = trim($this->_formulario->getDsordem());
            $this->_dsInicio = trim($this->_formulario->getDsinicio());
            $this->_dsFim = $this->_formulario->getDsfim();
            $this->_dtPagamento = $this->_formulario->getDtpagamento();
        }
    }


    /**
     * Enter description here ...
     *
     * @param integer $nrFormulario Numero do formulario.
     *
     * @throws Exception Propaga excecao.
     *
     * @return void
     */
    private function _buscaDadosFormularioFerias($nrFormulario){
        $formulario = new FpaFormferiasEntity();
        $this->_formulario = $formulario->retornaDadosFormularioFerias($this->_nrOrgTrabalho, $nrFormulario);
        if (is_object($this->_formulario) === true) {
            $this->_dsOrdem = $this->_formulario->getDsordem();
            $this->_dsInicio = trim($this->_formulario->getDsinicio());
            $this->_dsFim = trim($this->_formulario->getDsfim());
            $this->_dtPagamento = $this->_formulario->getDtpagamento();
            $this->_dt_inicio = new TkDateTime($this->_formulario->getDtinigozoini());
            $this->_dt_fim = new TkDateTime($this->_formulario->getDtinigozofim());
        }else
        tk_abort('É necessário salvar o formulário antes de executar uma ação.');
    }


    /**
     * Enter description here ...
     *
     * @param integer $nrFormulario Numero do formulario.
     *
     * @throws Exception Propaga excecao.
     *
     * @return void
     */
    private function _buscaDadosFormularioRescisao($nrFormulario){
        $formulario = new FpaformrescisaoEntity();
        $this->_formulario = $formulario->retornaDadosFormularioRescisao($this->_nrOrgTrabalho, $nrFormulario);
        if (is_object($this->_formulario) === true) {
            $this->_dsOrdem = $this->_formulario->getDsordem();
            $this->_dsInicio = trim($this->_formulario->getDsinicio());
            $this->_dsFim = trim($this->_formulario->getDsfim());
            $this->_dtPagamento = $this->_formulario->getDtpagamento();
            $this->_dt_inicio = new TkDateTime($this->_formulario->getDtrescisaoini());
            $this->_dt_fim = new TkDateTime($this->_formulario->getDtrescisaofim());
        }
    }

    /**
     * BORBA 25/02/2014 ...
     *
     * @param integer $nrFormulario Numero do formulario.
     *
     * @throws Exception Propaga excecao.
     *
     * @return void
     */
    private function _buscaDadosFormularioRescocomplementar($nrFormulario){

        $this->_formulario = \Folha\Entities\FpaFormresccomplementar::findOneByNrorgAndNrformrescisao($this->_nrOrgTrabalho, $nrFormulario);
        //tk_warning('calculo ordem-> '.$this->_formulario->getDsordem().' nomeini-> '.$this->_formulario->getDsinicio().' nomefim-> '.$this->_formulario->getDsfim().' dtpg-> '.$this->_formulario->getDtpagamento().' dtresc-> '.$this->_formulario->getDtrescisaoini().' dtrescfim-> '.$this->_formulario->getDtrescisaofim().' retrp-> '.$this->_formulario->getIdretrocesso() );
        if (is_object($this->_formulario) === true) {
            $this->_dsOrdem = $this->_formulario->getDsordem();
            $this->_dsInicio = $this->_formulario->getDsinicio();
            $this->_dsFim = $this->_formulario->getDsfim();
            $this->_dtPagamento = $this->_formulario->getDtpagamento();
            $this->_dt_inicio = new TkDateTime($this->_formulario->getDtrescisaoini());
            $this->_dt_fim = new TkDateTime($this->_formulario->getDtrescisaofim());
            //$this->_idRetrocesso = $this->_formulario->getIdretrocesso();
        }
    }

    /**
     * Executa preparacap para tipo de calculo de FERIAS / RESCISAO / PROVISAO CONTABIL.
     *
     * @param GpevinculomEntity $vinculo       Instancia de GpevinculomEntity.
     * @param Preparacao        $preparacao    Instancia de Preparacao.
     * @param Demonstrativo     $demonstrativo Instancia de Demonstrativo.
     * @param integer           $contador      Contador.
     *
     * @return void
     *
     * @author alessandrobartels 14/07/2011 - Criado novo metodo para diminuir a complexidade do percorre()
     */
    private function _executaPreparacao(
        GpevinculomEntity $vinculo, Preparacao $preparacao, Demonstrativo $demonstrativo, $contador
    ){
        if ($this->_nrTipoCalculo !== FpatipocalculoEntity::CALC_FOLHA &&
            $this->_nrTipoCalculo !== FpatipocalculoEntity::CALC_ADIANTAMENTO &&
            $this->_nrTipoCalculo !== FpatipocalculoEntity::CALC_FECHAMENTO
        ) {
            /* @todo Condicionar a execucao da preparacao a parametros na entrada de dados */
            $preparacao->preparar($vinculo, $this->_tp_calc_mov, $demonstrativo, $contador);
        }
    }

    
}